# do not import dios-stuff here


class OptsFields:
    """storage class for the keys in ``dios_options``"""

    """Set the number of rows and variables to display in a call that use
    ``__repr__`` or ``__str__`` like e.g. ``print(dios)`` do."""
    disp_max_rows = "disp_max_rows "
    disp_max_vars = "disp_max_vars"

    """
    none_plus: none or more columns, than in self, can be given
    at_least_one: accept if at least one column is present in self
    all_present: all given columns must be present in self
    """
    col_indexing_method = "col_indexing_method"

    mixed_itype_warn_policy = "mixed_itype_policy"

    """
    should nans be droppend during comparision(drop), 
    stay nans (keep), or be compared (nplike). 
    nplike is quite silly as `5 < NaN` will simply evaluate to False"""
    comparison_nan_policy = "comparison_nan_policy"

    """
    Get item nan policy:
    
    """


class Opts:
    none_or_more = 'none_more'
    one_or_more = 'one_more'
    all_or_more = 'all_more'
    none_up2_all = 'none_all'
    one_up2_all = 'one_all'
    exactly_all = 'all_all'

    itype_warn = 'warn'
    itype_err = 'err'
    itype_ignore = 'ignore'

    keep_nans = 'keep'
    nplike_nans = 'nplike'
    drop_nans = 'drop'


# set default values
dios_options = {
    OptsFields.disp_max_rows: 10,
    OptsFields.disp_max_vars: 4,
    OptsFields.col_indexing_method: Opts.none_or_more,
    OptsFields.mixed_itype_warn_policy: Opts.itype_ignore,
    OptsFields.comparison_nan_policy: Opts.keep_nans,
}


def check_keys_by_policy(tocheck, keys, policy):

    filtered = [k for k in tocheck if k in keys]
    if policy == Opts.none_up2_all:
        fail = [k for k in tocheck if k not in keys]
        if fail:
            raise KeyError(f"Policy says: keys must be known. Unknown: {fail}")

    elif policy == Opts.none_or_more:
        pass

    elif policy == Opts.one_up2_all:
        fail = [k for k in tocheck if k not in keys]
        if not filtered or fail:
            if fail:
                raise KeyError(f"Policy says: keys must be known and at least one must be shared. Unknown: {fail}")
            raise KeyError("Policy says: keys must known and at least one key must be shared. None was shared.")

    elif policy == Opts.one_or_more:
        if not filtered:
            raise KeyError("Policy says: at least one key must be shared.")

    elif policy == Opts.exactly_all:
        fail = set(tocheck).symmetric_difference(set(keys))
        if fail:
            raise KeyError(f"Policy says: exactly all keys must be given.")

    elif Opts.all_or_more:
        fail = set(filtered).symmetric_difference(set(keys))
        if fail:
            raise KeyError(f"Policy says: all known keys must be given, unknown are ignored.")
    else:
        raise ValueError(policy)

    return filtered
